# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022 CERN.
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Pytest configuration.

See https://pytest-invenio.readthedocs.io/ for documentation on which test
fixtures are available.
"""

import pytest
from invenio_access.permissions import system_identity
from invenio_requests.records.api import Request
from invenio_search import current_search

from invenio_communities.members.records.api import ArchivedInvitation, Member


#
# Function scope
#
@pytest.fixture(scope="function")
def clean_index(member_service, requests_service, db):
    """Clean the member and request index to match database state.

    Use this function when your tests depends on having a clean index
    state. This is an "expensive" fixture to run, thus only use it if your
    tests really doesn't work without.
    """
    list(current_search.delete(index_list=[
        Request.index._name,
        Member.index._name,
        ArchivedInvitation.index._name,
    ]))
    list(current_search.create(index_list=[
        Request.index._name,
        Member.index._name,
        ArchivedInvitation.index._name,
    ]))
    member_service.rebuild_index(system_identity)
    requests_service.rebuild_index(system_identity)
    Member.index.refresh()
    Request.index.refresh()
    ArchivedInvitation.index.refresh()
    return True


@pytest.fixture(scope="function")
def invite_user(member_service, community, owner, new_user, db):
    """An invited user."""
    data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
        "message": "Welcome to the club!",
    }
    member_service.invite(owner.identity, community._record.id, data)
    return new_user


@pytest.fixture(scope="function")
def public_reader(member_service, community, new_user, db):
    """Add a reader member with public visibility."""
    data = {
        "members": [{"type": "user", "id": str(new_user.id)}],
        "role": "reader",
        "visible": True,
    }
    member_service.add(system_identity, community._record.id, data)
    return new_user


@pytest.fixture(scope="function")
def invite_request_id(requests_service, invite_user):
    """An invitation request."""
    Request.index.refresh()
    res = requests_service.search(
        invite_user.identity,
        receiver={'user': invite_user.id},
        type='community-invitation',
    ).to_dict()
    return res['hits']['hits'][0]['id']



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test record - organization relationships."""

from copy import deepcopy
import pytest
from invenio_records.systemfields.relations import InvalidRelationValue
from jsonschema.exceptions import ValidationError

from invenio_communities.communities.records.api import Community


@pytest.fixture(scope="module")
def full_community(full_community):
    full_community.pop('slug')
    return full_community


#
# Tests
#
def test_organizations_field(app):
    """organizations should be defined as a relation."""
    assert 'organizations' in Community.relations
    assert Community.relations.organizations


def test_community_organizations_validation(
    app, db, location, full_community, community_type_record
):
    comm = Community.create(full_community, slug='test')
    comm.commit()
    db.session.commit()

    # test it was saved properly
    aff = comm.metadata["organizations"][0]
    assert aff["name"] == "My Org"


def test_community_organizations_with_multiple_organizations(
    app, db, location, full_community, community_type_record
):
    community_copy = deepcopy(full_community)
    community_copy["metadata"]["organizations"].append(
        {"name": "Another organization"}
    )
    comm = Community.create(community_copy, slug='test')
    comm.commit()
    db.session.commit()

    aff_list = comm.metadata["organizations"]
    assert len(aff_list) == 2
    assert aff_list[0]["name"] == "My Org"
    assert aff_list[1]["name"] == "Another organization"


def test_community_organizations_with_name_cleanup_validation(
    app, db, location, affiliation, minimal_community
):
    """Creates a community with an existing affiliation."""
    slug = minimal_community.pop('slug')
    community_copy = deepcopy(minimal_community)
    community_copy["metadata"]["organizations"] = [
        {"id": "cern", "name": "CERN"}
    ]
    comm = Community.create(community_copy, slug=slug)
    comm.commit()
    db.session.commit()

    # test it was saved properly
    aff = comm.metadata["organizations"][0]
    # name is not retrieved since it is only used when dereferencing
    assert aff.get("id") == "cern"
    assert aff.get("name") is None


def test_community_organizations_indexing(
    app, db, affiliation, location, full_community, community_type_record
):
    community_copy = deepcopy(full_community)
    community_copy["metadata"]["organizations"].append(
        {"id": "cern", "name": "CERN"}
    )
    comm = Community.create(community_copy, slug='test').commit()
    db.session.commit()

    # Dump community - dumps will dereference relations.
    dump = comm.dumps()
    expected_organisations = [
        {
            "name": "My Org"
        },
        {
            "id": "cern",
            "name": "CERN",
            "@v": dump["metadata"]["organizations"][1]["@v"]
        }
    ]

    assert dump["metadata"]["organizations"] == expected_organisations

    # Load comm again - should produce an identical record.
    loaded_comm = Community.loads(dump)
    assert dict(comm) == dict(loaded_comm)

    # Calling commit() will clear the dereferenced relation.
    loaded_comm.commit()
    loaded_aff_list = loaded_comm["metadata"]["organizations"]
    assert len(loaded_aff_list) == 2
    assert loaded_aff_list[0] == {"name": "My Org"}
    assert loaded_aff_list[1] == {"id": "cern"}


def test_community_organizations_invalid(
    app, db, location, full_community, community_type_record
):
    """Should fail on invalid id's and invalid structure."""
    comunity_copy = deepcopy(full_community)
    # The id "invalid" does not exists.
    comunity_copy["metadata"]["organizations"] = (
        [{"id": "invalid"}]
    )
    pytest.raises(InvalidRelationValue,
                  Community.create(comunity_copy, slug='test').commit)

    # Not a list of objects
    comunity_copy["metadata"]["organizations"] = {"id": "cern"}
    pytest.raises(
        ValidationError, Community.create, comunity_copy, slug='test')

    # no additional keys are allowed
    comunity_copy["metadata"]["organizations"] = (
        [{"test": "cern"}]
    )
    pytest.raises(
        ValidationError, Community.create, comunity_copy, slug='test')

    # non-string types are not allowed as id values
    comunity_copy["metadata"]["organizations"] = [{"id": 1}]
    pytest.raises(
        ValidationError, Community.create, comunity_copy, slug='test')

    # No duplicates
    comunity_copy["metadata"]["organizations"] = (
        [{"id": "cern"}, {"id": "cern"}]
    )
    pytest.raises(
        ValidationError, Community.create, comunity_copy, slug='test')



#
# This file is part of Invenio.
# Copyright (C) 2016-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Remove version table"""

import sqlalchemy as sa
import sqlalchemy_utils
from alembic import op
from sqlalchemy.dialects import mysql, postgresql

# revision identifiers, used by Alembic.
revision = 'a3f5a8635cbb'
down_revision = '5b478fe7ef7f'
branch_labels = ()
depends_on = None


def upgrade():
    """Upgrade database."""
    op.drop_index(
        'ix_communities_metadata_version_end_transaction_id',
        table_name='communities_metadata_version'
    )
    op.drop_index(
        'ix_communities_metadata_version_operation_type',
        table_name='communities_metadata_version'
    )
    op.drop_index(
        'ix_communities_metadata_version_transaction_id',
        table_name='communities_metadata_version'
    )
    op.drop_table('communities_metadata_version')
    op.add_column(
        'communities_metadata',
        sa.Column('slug', sa.String(length=255), nullable=True)
    )
    op.create_unique_constraint(
        op.f('uq_communities_metadata_slug'), 'communities_metadata', ['slug']
    )


def downgrade():
    """Downgrade database."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        op.f('uq_communities_metadata_slug'),
        'communities_metadata',
        type_='unique'
    )
    op.drop_column('communities_metadata', 'slug')
    op.create_table(
        "communities_metadata_version",
        sa.Column(
            "created",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "updated",
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            autoincrement=False,
            nullable=False,
        ),
        sa.Column(
            "json",
            sa.JSON()
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "mysql")
            .with_variant(
                postgresql.JSONB(none_as_null=True, astext_type=sa.Text()),
                "postgresql"
            )
            .with_variant(sqlalchemy_utils.types.json.JSONType(), "sqlite"),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "version_id", sa.Integer(), autoincrement=False, nullable=True),
        sa.Column(
            "bucket_id",
            sqlalchemy_utils.types.uuid.UUIDType(),
            autoincrement=False,
            nullable=True,
        ),
        sa.Column(
            "transaction_id",
            sa.BigInteger(),
            autoincrement=False,
            nullable=False
        ),
        sa.Column("end_transaction_id", sa.BigInteger(), nullable=True),
        sa.Column("operation_type", sa.SmallInteger(), nullable=False),
        sa.PrimaryKeyConstraint(
            "id",
            "transaction_id",
            name=op.f("pk_communities_metadata_version")
        ),
    )
    op.create_index(
        op.f("ix_communities_metadata_version_end_transaction_id"),
        "communities_metadata_version",
        ["end_transaction_id"],
        unique=False,
    )
    op.create_index(
        op.f("ix_communities_metadata_version_operation_type"),
        "communities_metadata_version",
        ["operation_type"],
        unique=False,
    )
    op.create_index(
        op.f("ix_communities_metadata_version_transaction_id"),
        "communities_metadata_version",
        ["transaction_id"],
        unique=False,
    )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN
#
# Invenio-Communities is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Test communities types relationships."""


#
# Tests
#

from copy import deepcopy
import pytest

from invenio_communities.communities.records.api import Community
from invenio_records.systemfields.relations.errors import InvalidRelationValue


@pytest.fixture(scope="module")
def minimal_community(minimal_community):
    minimal_community.pop('slug')
    return minimal_community


@pytest.mark.parametrize(
    "community_type, expected",
    [
        ("organization", "organization"),
        ("event", "event"),
        ("topic", "topic"),
        ("project", "project"),
    ],
)
def test_valid_community_types(
    db, location, minimal_community, community_type, expected,
    community_type_record
):
    community_copy = deepcopy(minimal_community)
    community_copy["metadata"].update({
        "type": {
            "id": community_type
        }
    })

    comm = Community.create(community_copy, slug='test')
    comm.commit()
    db.session.commit()

    assert comm.metadata["type"]["id"] == expected


def test_invalid_community_type(
    db, minimal_community, community_type_record
):
    community_copy = deepcopy(minimal_community)
    community_copy["metadata"].update({
        "type": {
            "id": "invalid_type"
        }
    })
    with pytest.raises(InvalidRelationValue):
        comm = Community.create(community_copy, slug='test')
        comm.commit()
        db.session.commit()


def test_organizations_field():
    """organizations should be defined as a relation."""
    assert "type" in Community.relations
    assert Community.relations.organizations



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
# Copyright (C)      2022 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community database models."""

from datetime import datetime

from invenio_db import db
from invenio_files_rest.models import Bucket
from invenio_records.models import RecordMetadataBase, Timestamp
from invenio_records_resources.records import FileRecordModelMixin
from sqlalchemy.dialects import mysql
from sqlalchemy.types import String
from sqlalchemy_utils.types import UUIDType


class CommunityMetadata(db.Model, RecordMetadataBase):
    """Represent a community."""

    __tablename__ = 'communities_metadata'

    slug = db.Column(String(255), unique=True, nullable=True)

    bucket_id = db.Column(UUIDType, db.ForeignKey(Bucket.id))
    bucket = db.relationship(Bucket)


class CommunityFileMetadata(db.Model, RecordMetadataBase, FileRecordModelMixin):
    """File associated with a community."""

    __record_model_cls__ = CommunityMetadata

    __tablename__ = 'communities_files'


class CommunityFeatured(db.Model, Timestamp):
    """Featured community entry."""

    __tablename__ = 'communities_featured'

    id = db.Column(db.Integer, primary_key=True)
    community_id = db.Column(
        UUIDType,
        db.ForeignKey(CommunityMetadata.id),
        nullable=False
    )
    start_date = db.Column(
        db.DateTime().with_variant(mysql.DATETIME(fsp=6), "mysql"),
        nullable=False,
        default=datetime.utcnow,
    )



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

from functools import wraps

from flask import g

from invenio_communities.proxies import current_communities


def pass_community(f):
    """Fetch the community record."""
    @wraps(f)
    def view(**kwargs):
        pid_value = kwargs["pid_value"]
        community = current_communities.service.read(
            id_=pid_value, identity=g.identity
        )
        kwargs["community"] = community
        return f(**kwargs)

    return view



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS bundles for communities.

You include one of the bundles in a page like the example below (using
``comunities`` bundle as an example):

.. code-block:: html

    {{ webpack['communities.js']}}

"""

from invenio_assets.webpack import WebpackThemeBundle

communities = WebpackThemeBundle(
    __name__,
    'assets',
    default='semantic-ui',
    themes={
        'semantic-ui': dict(
            entry={
                'invenio-communities-new':
                    './js/invenio_communities/community/new.js',
                'invenio-communities-privileges':
                    './js/invenio_communities/settings/privileges.js',
                'invenio-communities-profile':
                    './js/invenio_communities/settings/profile/index.js',
                'invenio-communities-requests':
                    './js/invenio_communities/requests/index.js',
                'invenio-communities-frontpage':
                    './js/invenio_communities/community/frontpage.js',
                'invenio-communities-details-search':
                    './js/invenio_communities/details_search/index.js',
                'invenio-communities-search':
                    './js/invenio_communities/community/search.js',
                'invenio-communities-members':
                    './js/invenio_communities/members/members/member_view/index.js',
                'invenio-communities-members-manager':
                    './js/invenio_communities/members/members/manager_view/index.js',
                'invenio-communities-members-public':
                    './js/invenio_communities/members/members/public_view/index.js',
                'invenio-communities-invitations':
                    './js/invenio_communities/members/invitations/index.js'
            },
            dependencies={
                '@ckeditor/ckeditor5-build-classic': '^16.0.0',
                '@ckeditor/ckeditor5-react': '^2.1.0',
                '@semantic-ui-react/css-patch': '^1.0.0',
                'react-router-dom': '^6.3.0',
                'react-invenio-forms': '^0.10.0',
                'react-invenio-deposit': '^0.19.0',
                'axios': '^0.21.0',
                'formik': '^2.1.0',
                "i18next": "^20.3.0",
                "i18next-browser-languagedetector": "^6.1.0",
                'lodash': '^4.17.0',
                'luxon': '^1.23.0',
                'path': '^0.12.0',
                'prop-types': '^15.7.0',
                'qs': '^6.9.0',
                'react': '^16.13.0',
                'react-dom': '^16.13.0',
                "react-i18next": "^11.11.0",
                'react-redux': '^7.2.0',
                'react-searchkit': '^2.0.0',
                'redux': '^4.0.0',
                'redux-thunk': '^2.3.0',
                'semantic-ui-css': '^2.4.0',
                'semantic-ui-react': '^2.1.0',
                'yup': '^0.28.0',
            },
            aliases={
                # Define Semantic-UI theme configuration needed by
                # Invenio-Theme in order to build Semantic UI (in theme.js
                # entry point). theme.config itself is provided by
                # cookiecutter-invenio-rdm.
                '@js/invenio_communities': 'js/invenio_communities',
                '@translations/invenio_communities': 'translations/invenio_communities'
            }
        ),
    }
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
# Copyright (C) 2022 Northwestern University.
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community resources tests."""

import copy
from io import BytesIO
from uuid import uuid4

from invenio_communities.communities.records.api import Community
from invenio_communities.members import Member


def _assert_single_item_response(response):
    """Assert the fields present on a single item response."""
    response_fields = response.json.keys()
    fields_to_check = [
        'created', 'id', 'links', 'metadata', 'slug', 'updated',
    ]

    for field in fields_to_check:
        assert field in response_fields


def _assert_optional_medatada_items_response(response):
    """Assert the fields present on the metadata"""
    metadata_fields = response.json['metadata'].keys()
    fields_to_check = [
        "description", "curation_policy", "page", "website", "organizations"
    ]

    for field in fields_to_check:
        assert field in metadata_fields


def _assert_optional_access_items_response(response):
    """Assert the fields present on the metadata"""
    access_fields = response.json['access'].keys()
    fields_to_check = [
       "visibility", "member_policy", "record_policy",
    ]

    for field in fields_to_check:
        assert field in access_fields


def _assert_single_item_search(response):
    """Assert the fields present on the search response """
    response_fields = response.json.keys()
    fields_to_check = [
        "aggregations", "hits", "links", "sortBy"
    ]

    for field in fields_to_check:
        assert field in response_fields


def _assert_error_fields_response(expected, response):
    """Assert the fields present in response error list and fields tested """
    error_fields = [item['field'] for item in response.json['errors']]

    for field in expected:
        assert field in error_fields


def _assert_error_messages_response(expected, response):
    """Assert the error messages present in response error list and expected error messages """
    error_messages_list = ['Must be one of: organization, event, topic, project.',
                           'Must be one of: public, private.',
                           'Must be one of: public, restricted.',
                           'Not empty string and less than 2000 characters allowed.',
                           'Not empty string and less than 250 characters allowed.',
                           'Not empty string and less than 100 characters allowed.'
                          ]
    error_messages = set([item['messages'][0] for item in response.json['errors']])

    assert expected == len(set(error_messages_list).intersection(error_messages))


def test_simple_flow(
    client, location, minimal_community, headers, owner, db, es_clear
):
    """Test a simple REST API flow."""
    client = owner.login(client)

    # Create a community
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)

    created_community = res.json
    id_ = created_community["id"]
    slug = created_community["slug"]

    # Read the community
    res = client.get(f'/communities/{id_}', headers=headers)
    assert res.status_code == 200
    assert res.json['metadata'] == \
        created_community['metadata']

    read_community = res.json
    Community.index.refresh()

    # Read the community via slug
    res = client.get(f'/communities/{slug}', headers=headers)
    assert res.status_code == 200
    assert res.json['id'] == id_

    # Search for created community
    res = client.get(
        f'/communities', query_string={'q': f'id:{id_}'}, headers=headers)
    assert res.status_code == 200
    assert res.json['hits']['total'] == 1
    assert res.json['hits']['hits'][0]['metadata'] == \
        created_community['metadata']

    # Update community
    data = copy.deepcopy(read_community)
    data["metadata"]["title"] = 'New title'
    res = client.put(
        f'/communities/{id_}', headers=headers, json=data)
    assert res.status_code == 200
    assert res.json['metadata']["title"] == 'New title'

    updated_community = res.json
    Community.index.refresh()

    # Search for updated commmunity
    res = client.get(
        f'/communities', query_string={'q': f'id:{id_}'}, headers=headers)
    assert res.status_code == 200
    assert res.json['hits']['total'] == 1
    assert res.json['hits']['hits'][0]['metadata'] == \
        updated_community['metadata']
    data = res.json['hits']['hits'][0]
    assert updated_community['metadata']['title'] == 'New title'

    # Delete community
    res = client.delete(f'/communities/{id_}', headers=headers)
    assert res.status_code == 204

    # Read again community, should return 404
    res = client.get(f'/communities/{id_}', headers=headers)
    assert res.status_code == 410
    assert res.json["message"] == "The record has been deleted."


def test_post_schema_validation(
    client, location, minimal_community, headers, owner, es_clear, db
):
    """Test the validity of community json schema"""
    client = owner.login(client)

    # Create a community
    res = client.post('/communities', headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    assert minimal_community['metadata'] == created_community['metadata']

    # Assert required fields
    assert created_community['metadata'] == {"title": "My Community"}
    assert created_community['access']['visibility'] == "public"

    # Assert required enums
    data = copy.deepcopy(minimal_community)
    data['metadata']['type'] = {'id': 'foobar', 'title': 'foobar'}
    res = client.post('/communities', headers=headers, json=data)
    assert res.status_code == 400

    data = copy.deepcopy(minimal_community)
    data['access']['visibility'] = 'foobar'
    res = client.post('/communities', headers=headers, json=data)
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    _assert_error_fields_response(set(['access.visibility']), res)
    _assert_error_messages_response(1, res)

    # Delete the community
    res = client.delete(f'/communities/{created_community["id"]}', headers=headers)
    assert res.status_code == 204


def test_post_metadata_schema_validation(
    client, location, minimal_community, headers, db, owner, es_clear
):
    """Test the validity of community metadata schema"""
    client = owner.login(client)

    # Alter paypload for each field for test
    data = copy.deepcopy(minimal_community)

    # Assert field size constraints  (id, title, description, curation policy, page)
    # ID max 100
    data["slug"] = "x" * 101
    res = client.post(
        '/communities', headers=headers, json=data
    )
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]['field'] == 'slug'
    #assert res.json["errors"][0]['messages'] == ['Not a valid string.']

    # Title max 250
    data["slug"] = "my_comm"
    data['metadata']['title'] =  "x" * 251
    res = client.post(
        '/communities', headers=headers, json=data
    )
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]['field'] == 'metadata.title'
    #assert res.json["errors"][0]['messages'] == ['Title is too long.']

    # Description max 2000
    data['metadata']['title'] = "New Title"
    data['metadata']['description'] =  "x" * 2001
    res = client.post(
        '/communities', headers=headers, json=data
    )
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]['field'] == 'metadata.description'
    #assert res.json["errors"][0]['messages'] == ['Description is too long.']

    # Curation policy max 2000
    data['metadata']['description'] = "basic description"
    data['metadata']['curation_policy'] =  "x" * 2001
    res = client.post(
        '/communities', headers=headers, json=data
    )
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]['field'] == 'metadata.curation_policy'
    #assert res.json["errors"][0]['messages'] == ['Curation policy is too long.']

    # Curation policy max 2000
    data['metadata']['curation_policy'] = "no policy"
    data['metadata']['page'] = "".join([str(i) for i in range(2001)])
    res = client.post(
        '/communities', headers=headers, json=data
    )
    assert res.status_code == 400
    assert res.json["message"] == "A validation error occurred."
    assert res.json["errors"][0]['field'] == 'metadata.page'
    #assert res.json["errors"][0]['messages'] == ['Page is too long.']

    data['metadata']['page'] = "basic page"
    res = client.post(
        '/communities', headers=headers, json=data
    )
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)

    # Delete the community
    res = client.delete(f'/communities/{data["slug"]}', headers=headers)
    assert res.status_code == 204


def test_post_community_with_existing_id(
    client, location, minimal_community, headers, db, es_clear, owner
):
    """Test create two communities with the same id"""
    client = owner.login(client)

    # Create a community
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community['id']

    #Creta another community with the same id
    minimal_community['id'] = id_
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    assert res.status_code == 400
    assert res.json['errors'][0]['messages'][0] == \
        'A community with this identifier already exists.'

    # Delete the community
    res = client.delete(f'/communities/{id_}', headers=headers)
    assert res.status_code == 204


def test_post_community_with_deleted_id(
    client, location, minimal_community, headers, db, es_clear, owner
):
    """Test create a community with a deleted id"""
    client = owner.login(client)

    #Create a community
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community['id']

    # Delete community
    res = client.delete(f'/communities/{id_}', headers=headers)
    assert res.status_code == 204

    # Create another community with the same id
    minimal_community['id'] = id_
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    assert res.status_code == 400
    assert res.json['errors'][0]['messages'][0] == \
        'A community with this identifier already exists.'


def test_post_self_links(
    client, location, minimal_community, headers, db, es_clear, owner
):
    """Test self links generated after post"""
    client = owner.login(client)

    #Create a community
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community['id']
    slug  = created_community['slug']
    # assert '/'.join(created_community['links']['self'].split('/')[-2:]) == f'communities/{id_}'
    assert created_community['links']['self'] == f'https://127.0.0.1:5000/api/communities/{id_}'
    assert created_community['links']['self_html'] == f'https://127.0.0.1:5000/communities/{slug}'

    # Delete the community
    res = client.delete(f'/communities/{id_}', headers=headers)
    assert res.status_code == 204


def test_simple_search_response(
    client, location, minimal_community, owner, headers, es_clear,
    fake_communities, community_types
):
    """Test get/list and search functionality"""
    client = owner.login(client)

    community_copy = copy.deepcopy(minimal_community)
    community_copy['metadata']['type'] = community_types[-1]

    # Create many communities,
    slug_oldest, slug_newest, num_each, total = fake_communities

    # Search for any commmunity, default order newest
    res = client.get(
        f'/communities', query_string={'q': f''}, headers=headers)
    assert res.status_code == 200
    _assert_single_item_search(res)
    assert res.json['hits']['total'] == total
    assert res.json['hits']['hits'][0]['slug'] == slug_newest
    assert res.json['hits']['hits'][0]['metadata'] == community_copy['metadata']

    # Search filter for the second commmunity
    res = client.get(
        f'/communities', query_string={'q': 'metadata.type.id:project'},
        headers=headers)
    assert res.status_code == 200
    _assert_single_item_search(res)
    assert res.json['hits']['total'] == num_each

    # Sort results by oldest
    res = client.get(
        f'/communities', query_string={'q': '', 'sort': 'oldest'},
        headers=headers)
    assert res.status_code == 200
    assert res.json['hits']['hits'][0]['slug'] == slug_oldest

    # Test for page and size
    res = client.get(
        f'/communities', query_string={'q': '', 'size': '5', 'page': '2'},
        headers=headers)
    assert res.status_code == 200
    assert res.json['hits']['total'] == total
    assert len(res.json['hits']['hits']) == 5


def test_simple_get_response(
    client, location, full_community, headers, db, es_clear, owner,
    community_type_record
):
    """Test get response json schema"""
    client = owner.login(client)
    community_copy = copy.deepcopy(full_community)
    # Dereferenced community has attribute 'title' in 'type'
    community_copy['metadata']['type'].update({'title': {'en': 'Event'}})

    # Create a community
    res = client.post(
        '/communities', headers=headers, json=full_community
    )
    Member.index.refresh()
    assert res.status_code == 201
    _assert_single_item_response(res)
    _assert_optional_medatada_items_response(res)

    created_community = res.json

    # assert full_community['access'] == created_community['access']

    assert community_copy['metadata'] == created_community['metadata']
    id_ = created_community["id"]

    # Read the community
    res = client.get(f'/communities/{id_}', headers=headers)
    assert res.status_code == 200
    _assert_single_item_response(res)
    _assert_optional_medatada_items_response(res)
    # _assert_optional_access_items_response(res)
    # assert full_community['access'] == res.json['access']
    assert community_copy['metadata'] == res.json['metadata']

    # Read a non-existed community
    res = client.get(f'/communities/{id_[:-1]}', headers=headers)
    assert res.status_code == 404
    # assert res.message == 'The persistent identifier does not exist.'


def test_simple_put_response(
    client, location, minimal_community, headers, db, es_clear, owner,
    community_type_record
):
    """Test put response basic functionality"""
    client = owner.login(client)
    # Create a community
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community['id']
    Member.index.refresh()

    data = copy.deepcopy(minimal_community)
    data["metadata"] = \
    {
        "title": "New Community",
        "description": "This is an example Community.",
        "type": {
            "id": "event",
        },
        "curation_policy": "This is the kind of records we accept.",
        "page": "Information for my community.",
        "website": "https://inveniosoftware.org/",
    }

    data["access"] = \
    {
        "visibility": "restricted",
        "member_policy": "closed",
        "record_policy": "closed"
    }

    res = client.put(f'/communities/{id_}', headers=headers, json=data)
    # Dereferenced community has attribute 'title' in 'type'
    data["metadata"]["type"].update({"title": {"en": "Event"}})
    assert res.status_code == 200
    assert res.json['id'] == id_
    assert res.json['metadata'] == data["metadata"]
    # assert access_== data["access"]
    assert res.json["revision_id"] == int(created_community["revision_id"])+1

    # Update non-existing community
    res = client.put(
        f'/communities/{id_[:-1]}', headers=headers, json=data
    )
    assert res.status_code == 404
    assert res.json['message'] == 'The persistent identifier does not exist.'

    # Update deleted community
    res = client.delete(f'/communities/{id_}', headers=headers)
    assert res.status_code == 204
    data['metadata']['title'] =  "Deleted Community",
    res = client.put(f'/communities/{id_}', headers=headers, json=data)
    assert res.status_code == 410
    assert res.json['message']  == 'The record has been deleted.'


def test_update_renamed_record(
    client, location, minimal_community, headers, db, es_clear, owner
):
    """Test to update renamed entity."""
    client = owner.login(client)
    # Create a community
    res = client.post('/communities', headers=headers, json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community['id']
    slug = created_community['slug']

    # Rename the community
    data = copy.deepcopy(minimal_community)
    data['slug'] = "renamed_comm"
    res = client.post(f'/communities/{id_}/rename', headers=headers, json=data)
    assert res.status_code == 200
    renamed_community = res.json
    renamed_id_ = renamed_community['id']
    data["access"] = \
        {
            "visibility": "restricted",
            "member_policy": "closed",
            "record_policy": "closed"
        }
    res = client.put(f'/communities/{renamed_id_}', headers=headers, json=data)
    assert res.status_code == 200
    assert res.json['id'] == renamed_id_
    assert res.json['metadata'] == data["metadata"]
    assert res.json["revision_id"] == int(renamed_community["revision_id"])+1


def test_simple_delete_response(
    client, location, minimal_community, headers, db, es_clear, owner
):
    """Test delete and request deleted community."""
    client = owner.login(client)

    # Create a community
    res = client.post(
        '/communities', headers=headers,
        json=minimal_community)
    Member.index.refresh()

    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community['id']

    # Delete community
    res = client.delete(f'/communities/{id_}', headers=headers)
    assert res.status_code == 204

    # Read the community
    res = client.get(f'/communities/{id_}', headers=headers)
    assert res.status_code == 410
    assert res.json['message']  == 'The record has been deleted.'

    # Delete non-existing community
    res = client.delete(f'/communities/{id_[:-1]}', headers=headers)
    assert res.status_code == 404
    assert res.json['message']  == 'The persistent identifier does not exist.'


def test_logo_flow(
    app, client, location, minimal_community, headers, db, es_clear, owner
):
    """Test logo workflow."""
    client = owner.login(client)

    # Create a community
    res = client.post('/communities', headers=headers, json=minimal_community)
    assert res.status_code == 201
    created_community = res.json
    id_ = created_community['id']
    assert created_community['links']['logo'] == \
        f'https://127.0.0.1:5000/api/communities/{id_}/logo'
    Member.index.refresh()

    # Get non-existent logo
    res = client.get(f'/communities/{id_}/logo')
    assert res.status_code == 404
    assert res.json['message'] == 'No logo exists for this community.'

    # Delete non-existent logo
    res = client.delete(f'/communities/{id_}/logo', headers=headers)
    assert res.status_code == 404
    assert res.json['message'] == 'No logo exists for this community.'

    # Update logo
    res = client.put(
        f'/communities/{id_}/logo',
        headers={
            **headers,
            'content-type': 'application/octet-stream',
        },
        data=BytesIO(b'logo'),
    )
    assert res.status_code == 200
    assert res.json['size'] == 4

    # Get logo
    res = client.get(f'/communities/{id_}/logo')
    assert res.status_code == 200
    assert res.data == b'logo'

    # Update logo again
    res = client.put(
        f'/communities/{id_}/logo',
        headers={
            **headers,
            'content-type': 'application/octet-stream',
        },
        data=BytesIO(b'new_logo'),
    )
    assert res.status_code == 200
    assert res.json['size'] == 8

    # Get new logo
    res = client.get(f'/communities/{id_}/logo')
    assert res.status_code == 200
    assert res.data == b'new_logo'

    # Delete logo with unauthorized user
    with app.test_client() as anon_client:
        res = anon_client.delete(f'/communities/{id_}/logo', headers=headers)
        assert res.status_code == 403

    # Delete logo
    res = client.delete(f'/communities/{id_}/logo', headers=headers)
    assert res.status_code == 204

    # Try to get deleted logo
    res = client.get(f'/communities/{id_}/logo')
    assert res.status_code == 404
    assert res.json['message'] == 'No logo exists for this community.'

    # TODO: Delete community and try all of the above operations


def test_invalid_community_ids_create(
    client, location, minimal_community, headers, db, es_clear, owner
):
    client = owner.login(client)
    # Create a community with invalid ID
    minimal_community['slug'] = 'comm id'
    res = client.post('/communities', headers=headers, json=minimal_community)
    assert res.status_code == 400
    assert res.json['errors'][0]['messages'][0] == \
        'The ID should contain only letters with numbers or dashes.'


def test_invalid_community_ids(
    client, location, minimal_community, headers, db, es_clear, owner
):
    """Test for invalid community IDs handling."""
    client = owner.login(client)

    minimal_community['slug'] = 'comm_id'
    res = client.post('/communities', headers=headers, json=minimal_community)
    assert res.status_code == 201
    _assert_single_item_response(res)
    created_community = res.json
    id_ = created_community['id']
    Member.index.refresh()

    # Rename the community
    data = copy.deepcopy(minimal_community)
    data['slug'] = "Renamed comm"
    res = client.post(f'/communities/{id_}/rename', headers=headers, json=data)
    assert res.status_code == 400
    assert res.json['errors'][0]['messages'][0] == \
        'The ID should contain only letters with numbers or dashes.'

    data = copy.deepcopy(minimal_community)
    data = {}
    res = client.post(f'/communities/{id_}/rename', headers=headers, json=data)
    assert res.status_code == 400
    assert res.json['errors'][0]['messages'][0] == \
        'Missing data for required field.'

    new_id = "Renamed_comm"
    data['slug'] = new_id
    res = client.post(f'/communities/{id_}/rename', headers=headers, json=data)
    assert res.status_code == 200
    assert res.json['slug'] == new_id.lower()

    # UUID is invalid
    minimal_community['slug'] = str(uuid4())
    res = client.post('/communities', headers=headers, json=minimal_community)
    assert res.status_code == 400



"""Example of a record draft API."""

from invenio_records.systemfields import ConstantField
from invenio_records_resources.records import Record as RecordBase
from invenio_records_resources.records.systemfields import IndexField

from invenio_communities.records.records.systemfields import CommunitiesField

from .models import MockRecordMetadata, MockRecordCommunity


class MockRecord(RecordBase):
    """Example parent record."""

    # Configuration
    model_cls = MockRecordMetadata

    # System fields
    schema = ConstantField('$schema', 'local://mocks/mock-v1.0.0.json')

    index = IndexField('mocks-mock-v1.0.0', search_alias='mocks')

    communities = CommunitiesField(MockRecordCommunity)

    # request = CommunityRequestField(model=MockRequestCommunity)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Facet definitions."""

from flask_babelex import gettext as _
from invenio_records_resources.services.records.facets import TermsFacet

type = TermsFacet(
    field='metadata.type',
    label=_('Type'),
    value_labels={
        'organization': _('Organization'),
        'event': _('Event'),
        'topic': _('Topic'),
        'project': _('Project'),
    }
)

visibility = TermsFacet(
    field='access.visibility',
    label=_('Visibility'),
    value_labels={
        'public': _('Public'),
        'restricted': _('Restricted'),
    }
)

role = TermsFacet(
    field='role',
    label=_('Visibility'),
    value_labels={
        'owner': _('Owner'),
        'reader': _('Reader'),
        'manager': _('Manager'),
        'curator': _('Curator'),
    }
)

visible = TermsFacet(
    field='visible',
    label=_('Visibility'),
    value_labels={
        'hidden': _('Hidden'),
        'restricted': _('Restricted'),
    }
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
# Copyright (C) 2022 Northwestern University.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Community Service API."""

from .resources import CommunityResource, CommunityResourceConfig
from .services import CommunityFileServiceConfig, CommunityService, \
    CommunityServiceConfig, SearchOptions

__all__ = (
    'CommunityService',
    'CommunityServiceConfig',
    'CommunityFileServiceConfig',
    'CommunityResource',
    'CommunityResourceConfig',
    'SearchOptions',
)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2022 CERN.
# Copyright (C) 2022 Northwestern University.
# Copyright (C) 2022 Graz University of Technology.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

from flask_babelex import lazy_gettext as _
from invenio_access.permissions import system_identity, system_process
from invenio_db import db
from invenio_oaiserver.models import OAISet
from invenio_pidstore.errors import PIDDeletedError, PIDDoesNotExistError
from invenio_records_resources.services.records.components import \
    ServiceComponent
from marshmallow.exceptions import ValidationError

from ...proxies import current_roles
from ...utils import on_membership_change


class PIDComponent(ServiceComponent):
    """Service component for Community PIDs."""

    def set_slug(self, record, slug):
        error = ValidationError(
            _('A community with this identifier already exists.'),
            field_name='slug'
        )
        try:
            self.service.record_cls.pid.resolve(slug)
            raise error
        except PIDDeletedError:
            raise error
        except PIDDoesNotExistError:
            record.slug = slug

    def create(self, identity, record=None, data=None, **kwargs):
        """Create a Community PID from its metadata."""
        self.set_slug(record, data['slug'])

    def update(self, identity, record=None, data=None, **kwargs):
        """Rename the Community PIDs value."""
        if 'slug' in data and record.slug != data['slug']:
            raise ValidationError(
                _('The ID should be modified through the renaming URL instead.'),
                field_name='slug',
            )

    def rename(self, identity, record=None, data=None, **kwargs):
        """Rename the Community PIDs value."""
        if record.slug == data['slug']:
            raise ValidationError(
                _('A new ID value is required for the renaming.'),
                field_name='slug',
            )
        self.set_slug(record, data['slug'])

class CommunityAccessComponent(ServiceComponent):
    """Service component for access integration."""

    def _populate_access_and_validate(self, identity, data, record, **kwargs):
        """Populate and validate the community's access field."""
        if record is not None and "access" in data:
            # populate the record's access field with the data already
            # validated by marshmallow
            record.setdefault('access', {})
            record['access'].update(data.get("access", {}))
            record.access.refresh_from_dict(record.get("access"))

    def create(self, identity, data=None, record=None, **kwargs):
        """Add basic ownership fields to the record."""
        self._populate_access_and_validate(identity, data, record, **kwargs)

    def update(self, identity, data=None, record=None, **kwargs):
        """Update handler."""
        self._populate_access_and_validate(identity, data, record, **kwargs)


class OwnershipComponent(ServiceComponent):
    """Service component for owner membership integration."""

    def create(self, identity, data=None, record=None, **kwargs):
        """Make an owner member from the identity."""
        if system_process in identity.provides:
            return

        member = {
            "type": "user",
            "id": str(identity.id),
        }
        self.service.members.add(
            # the user is not yet owner of the community (is being added)
            # therefore we cannot use `identity`
            system_identity,
            record.id,
            {"members": [member], "role": current_roles.owner_role.name},
            uow=self.uow,
        )

        # Invalidate the membership cache
        on_membership_change(identity=identity)


class FeaturedCommunityComponent(ServiceComponent):
    """Service component for featured community integration."""

    def featured_create(self, identity, data=None, record=None, **kwargs):
        if(record.access.visibility != "public"):
            raise ValidationError(
                _("The community is not public"),
                field_name="community_id"
            )


class OAISetComponent(ServiceComponent):
    """Service component for OAI set integration."""

    _oai_sets_prefix = "community"

    def _retrieve_set(self, slug):
        return OAISet.query.filter(OAISet.spec == self._create_set_spec(slug)).first()

    def _create_set_spec(self, community_slug):
        return "{prefix}-{slug}".format(prefix=self._oai_sets_prefix, slug=community_slug)

    def _create_set_description(self, community_title):
        # NOTE: Does not require translation since this description can also be changed by an admin
        return "Records belonging to the community '{title}'".format(title=community_title)

    def _create_set_from_community(self, record):
        community_slug = record.slug
        community_title = record.metadata["title"]

        community_set = OAISet()
        community_set.name = community_title
        community_set.description = self._create_set_description(community_title)
        community_set.spec = self._create_set_spec(community_slug)
        community_set.search_pattern = "parent.communities.ids:{id}".format(id=record.id)

        return community_set

    def create(self, identity, data=None, record=None, **kwargs):
        """Create a set for the community."""
        if record.access.visibility != "public":
            return

        community_set = self._create_set_from_community(record)
        # NOTE: will be indexed via a listener in oaiserver module
        db.session.add(community_set)


    def delete(self, identity, data=None, record=None, **kwargs):
        """Remove the set for the community."""
        community_set = self._retrieve_set(kwargs.get("slug") or record.slug)
        if not community_set:
            return

        # NOTE: will be removed from index via listener in oaiserver module
        db.session.delete(community_set)


    def update(self, identity, data=None, record=None, **kwargs):
        """Update set accordingly."""
        if record.access.visibility != "public":
            # delete set if the community is not public anymore
            self.delete(identity, data, record, **kwargs)
            return

        community_set = self._retrieve_set(record.slug)
        if not community_set:
            # create new set because community is now public
            self.create(identity, data, record, **kwargs)
            return

        # just update set with new name and description
        community_set.name = record.metadata["title"]
        community_set.description = self._create_set_description(community_set.name)
        db.session.add(community_set)


    def rename(self, identity, record=None, data=None, old_slug=None, **kwargs):
        """Delete old set and create new set on ID change"""
        if record.access.visibility != "public":
            return

        # Since we want to delete the set created with the old set, we can not
        # use the already updated pid_value
        self.delete(identity, data=data, record=record, **kwargs, slug=old_slug)
        community_set = self._create_set_from_community(record)
        db.session.add(community_set)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# Invenio-RDM-Records is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Result items for OAI-PMH services."""

from invenio_records_resources.pagination import Pagination
from invenio_records_resources.services.base.results import ServiceListResult


# TODO: Move to invenio-records-resources together with OAISetList in
#       invenio-rdm-records and import from there
class BaseServiceListResult(ServiceListResult):
    """List of result items."""

    def __init__(
        self,
        service,
        identity,
        results,
        params=None,
        links_tpl=None,
        links_item_tpl=None,
        schema=None,
    ):
        """Constructor.

        :params service: a service instance
        :params identity: an identity that performed the service request
        :params results: the db search results
        :params params: dictionary of the query parameters
        """
        self._identity = identity
        self._results = results
        self._service = service
        self._schema = schema or service.schema
        self._params = params
        self._links_tpl = links_tpl
        self._links_item_tpl = links_item_tpl

    def __len__(self):
        """Return the total numer of hits."""
        return self.total

    def __iter__(self):
        """Iterator over the hits."""
        return self.hits

    @property
    def total(self):
        """Get total number of hits."""
        return self._results.total

    @property
    def hits(self):
        """Iterator over the hits."""
        for hit in self._results.items:
            # Project the record
            projection = self._schema.dump(
                hit,
                context=dict(
                    identity=self._identity,
                ),
            )
            if self._links_item_tpl:
                projection['links'] = self._links_item_tpl.expand(hit)

            yield projection

    @property
    def pagination(self):
        """Create a pagination object."""
        return Pagination(
            self._params['size'],
            self._params['page'],
            self.total,
        )

    def to_dict(self):
        """Return result as a dictionary."""
        # TODO: This part should imitate the result item above. I.e. add a
        # "data" property which uses a ServiceSchema to dump the entire object.

        res = {
            "hits": {
                "hits": list(self.hits),
                "total": self.total,
            }
        }

        if self._params:
            if self._links_tpl:
                res['links'] = self._links_tpl.expand(self.pagination)

        return res


class CommunityFeaturedList(BaseServiceListResult):
    """List of featured community entry result items."""




#
# This file is part of Invenio.
# Copyright (C) 2022 Northwestern University.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Create communities members table."""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import mysql, postgresql
from sqlalchemy_utils.types import JSONType, UUIDType

# revision identifiers, used by Alembic.
revision = 'f701a32e6fbe'
down_revision = 'de9c14cbb0b2'
branch_labels = ()
depends_on = '9848d0149abd'


def upgrade():
    """Upgrade database."""
    op.create_table(
        'communities_members',
        sa.Column(
            'created',
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), 'mysql'),
            nullable=False
        ),
        sa.Column(
            'updated',
            sa.DateTime().with_variant(mysql.DATETIME(fsp=6), 'mysql'),
            nullable=False
        ),
        sa.Column(
            'json',
            sa.JSON()
            .with_variant(JSONType(), 'mysql')
            .with_variant(
                postgresql.JSONB(none_as_null=True, astext_type=sa.Text()),
                'postgresql'
            )
            .with_variant(JSONType(), 'sqlite'),
            nullable=True
        ),
        sa.Column('version_id', sa.Integer(), nullable=False),
        sa.Column('id', UUIDType(), nullable=False),
        sa.Column('community_id', UUIDType(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=True),
        sa.Column('role', sa.String(length=50), nullable=False),
        sa.ForeignKeyConstraint(
            ['community_id'],
            ['communities_metadata.id'],
            name=op.f(
                'fk_communities_members_community_id_communities_metadata'
            ),
            ondelete='CASCADE'
        ),
        sa.ForeignKeyConstraint(
            ['user_id'],
            ['accounts_user.id'],
            name=op.f(
                'fk_communities_members_user_id_accounts_user'
            ),
            ondelete='CASCADE'
        ),
        sa.PrimaryKeyConstraint('id', name=op.f('pk_communities_members'))
    )
    op.create_index(
        'ix_community_user',
        'communities_members',
        ['community_id', 'user_id'],
        unique=True
    )


def downgrade():
    """Downgrade database."""
    op.drop_index('ix_community_user', table_name='communities_members')
    op.drop_table('communities_members')
